@@ -1,10 +1,10 @@
 import argparse
 import asyncio
-from scraper2_hj3415.krx300.krx300 import sync_with_mongo
+from ..krx300.krx300 import sync_with_mongo
 
 def main():
     parser = argparse.ArgumentParser(description="Krx300 CLI")
-    subparsers = parser.add_subparsers(dest='command', required=True)
+    subparsers = parser.add_subparsers(dest='command', help='명령어', required=True)
 
     # 'sync' 명령어 추가
     subparsers.add_parser('sync', help='몽고db와 krx300의 싱크를 맞춥니다.')
