@@ -1,17 +1,19 @@
 import asyncio
 import random
+
+import numpy as np
 import pandas as pd
 from io import StringIO
-from collections.abc import Callable, Awaitable
-from playwright.async_api import async_playwright, Page, Locator, TimeoutError as PlaywrightTimeoutError
+from playwright.async_api import Page, Locator, TimeoutError as PlaywrightTimeoutError
 from utils_hj3415 import setup_logger, tools
-from scraper2_hj3415.scraper import helper
-
+from . import helper
+from db2_hj3415.nfs import C101, C103, C104, C106, C108, 항목값y, 항목값q, 기업데이터
+from datetime import datetime, timezone
 
 mylogger = setup_logger(__name__, 'WARNING')
 
 
-async def parse_c101(code: str, page: Page) -> dict | None:
+async def parse_c101(code: str, page: Page) -> C101 | None:
     url = f"https://navercomp.wisereport.co.kr/v2/company/c1010001.aspx?cmp_cd={code}"
 
     page.set_default_timeout(3000)
@@ -79,7 +81,7 @@ async def parse_c101(code: str, page: Page) -> dict | None:
     개요_list = [(await li.text_content()).strip() for li in li_elements]
     개요 = "".join(개요_list)
 
-    return {
+    data = {
         "종목명": 종목명,
         "코드": 코드,
         "날짜": 날짜,
@@ -109,6 +111,25 @@ async def parse_c101(code: str, page: Page) -> dict | None:
         "개요": 개요,
     }
 
+    mylogger.debug(data)
+
+    return C101(**data)
+
+async def parse_c101_many(codes: list[str], page:Page) -> list[C101|None]:
+    results: list[C101|None] = []
+
+    for code in codes:
+        print(f"Parsing.. {code} / c101")
+        try:
+            result = await parse_c101(code, page)
+            results.append(result)
+        except Exception as e:
+            mylogger.error(f"Error while parsing {code}: {e}")
+            results.append(None)
+        finally:
+            await asyncio.sleep(random.uniform(1.0, 2.0))
+    return results
+
 async def click_buttons(page: Page, buttons: list[tuple[str, str]]) -> bool:
     """
     Playwright에서 버튼 클릭을 처리하는 비동기 함수
@@ -134,7 +155,27 @@ async def click_buttons(page: Page, buttons: list[tuple[str, str]]) -> bool:
     mylogger.debug('*** Buttons click done ***')
     return True
 
-async def parse_c103(code: str, page: Page) -> dict[str, pd.DataFrame]:
+T_C1034 = list[항목값y] | list [항목값q]
+
+def df_to_c1034_model(data: dict[str, pd.DataFrame]) -> dict[str, T_C1034]:
+    cleaned_data: dict[str, T_C1034] = {}
+
+    for page, df in data.items():
+        if not isinstance(df, pd.DataFrame):
+            continue
+
+        records = df.replace({np.nan: None}).to_dict(orient="records")
+
+        if page.endswith("q"):
+            cleaned_data[page] = [항목값q(**item) for item in records]
+        elif page.endswith("y"):
+            cleaned_data[page] = [항목값y(**item) for item in records]
+        else:
+            raise ValueError(f"알 수 없는 페이지 키 형식: {page}")
+
+    return cleaned_data
+
+async def parse_c103(code: str, page: Page) -> C103:
     '''
     # XPATH 상수
     손익계산서 = '//*[@id="rpt_tab1"]'
@@ -186,6 +227,8 @@ async def parse_c103(code: str, page: Page) -> dict[str, pd.DataFrame]:
     mylogger.debug(f"페이지 제목: {await page.title()}")
     await asyncio.sleep(2)
 
+    data = {'코드':code, '날짜':datetime.now(timezone.utc)}
+
     dfs:dict[str, pd.DataFrame] = {}
 
     for title, btn_list in btns.items():
@@ -200,9 +243,27 @@ async def parse_c103(code: str, page: Page) -> dict[str, pd.DataFrame]:
         except Exception as e:
             mylogger.error(f"{title} 테이블 파싱 실패: {e}")
             dfs[title] = pd.DataFrame()
-    return dfs
 
-async def parse_c104(code: str, page: Page) -> dict[str, pd.DataFrame]:
+    data.update(df_to_c1034_model(dfs))
+    return C103(**data)
+
+async def parse_c103_many(codes: list[str], page:Page) -> dict[str, C103 | None]:
+    results: dict[str, C103 | None] = {}
+
+    for code in codes:
+        print(f"Parsing.. {code} / c103")
+        try:
+            result = await parse_c103(code, page)
+            results[code] = result
+        except Exception as e:
+            mylogger.error(f"Error while parsing {code}: {e}")
+            results[code] = None
+        finally:
+            await asyncio.sleep(random.uniform(1.0, 2.0))
+    return results
+
+
+async def parse_c104(code: str, page: Page) -> C104:
     '''
     # XPATH 상수
     수익성 = '//*[ @id="val_tab1"]'
@@ -278,6 +339,8 @@ async def parse_c104(code: str, page: Page) -> dict[str, pd.DataFrame]:
     mylogger.debug(f"페이지 제목: {await page.title()}")
     await asyncio.sleep(2)
 
+    data = {'코드':code, '날짜':datetime.now(timezone.utc)}
+
     dfs: dict[str, pd.DataFrame] = {}
 
     for title, btn_list in btns.items():
@@ -297,9 +360,38 @@ async def parse_c104(code: str, page: Page) -> dict[str, pd.DataFrame]:
             mylogger.error(f"{title} 테이블 파싱 실패: {e}")
             dfs[title] = pd.DataFrame()
 
-    return dfs
+    data.update(df_to_c1034_model(dfs))
+    return C104(**data)
 
-async def parse_c106(code: str, page: Page) -> dict[str, pd.DataFrame]:
+async def parse_c104_many(codes: list[str], page:Page) -> dict[str, C104 | None]:
+    results: dict[str, C104 | None] = {}
+
+    for code in codes:
+        print(f"Parsing.. {code} / c104")
+        try:
+            result = await parse_c104(code, page)
+            results[code] = result
+        except Exception as e:
+            mylogger.error(f"Error while parsing {code}: {e}")
+            results[code] = None
+        finally:
+            await asyncio.sleep(random.uniform(1.0, 2.0))
+    return results
+
+
+def df_to_c106_model(data: dict[str, pd.DataFrame]) -> dict[str, list[기업데이터]]:
+    cleaned_data: dict[str, list[기업데이터]] = {}
+
+    for page, df in data.items():
+        if not isinstance(df, pd.DataFrame):
+            continue
+
+        records = df.replace({np.nan: None}).to_dict(orient="records")
+        cleaned_data[page] = [기업데이터(**item) for item in records]
+
+    return cleaned_data
+
+async def parse_c106(code: str, page: Page) -> C106:
     async def get_company_names_from_header() -> list:
         selector = '#cTB611_h'
         await page.wait_for_selector(selector, state="attached")
@@ -363,6 +455,8 @@ async def parse_c106(code: str, page: Page) -> dict[str, pd.DataFrame]:
     mylogger.debug(f"페이지 제목: {await page.title()}")
     await asyncio.sleep(2)
 
+    data = {'코드': code, '날짜': datetime.now(timezone.utc)}
+
     dfs: dict[str, pd.DataFrame] = {}
 
     company_names = await get_company_names_from_header()
@@ -380,9 +474,25 @@ async def parse_c106(code: str, page: Page) -> dict[str, pd.DataFrame]:
             mylogger.error(f"{period} 테이블 파싱 실패: {e}")
             dfs[period.lower()] = pd.DataFrame()
 
-    return dfs
+    data.update(df_to_c106_model(dfs))
+    return C106(**data)
 
-async def parse_c108(code: str, page: Page) -> pd.DataFrame:
+async def parse_c106_many(codes: list[str], page:Page) -> dict[str, C106 | None]:
+    results: dict[str, C106 | None] = {}
+
+    for code in codes:
+        print(f"Parsing.. {code} / c106")
+        try:
+            result = await parse_c106(code, page)
+            results[code] = result
+        except Exception as e:
+            mylogger.error(f"Error while parsing {code}: {e}")
+            results[code] = None
+        finally:
+            await asyncio.sleep(random.uniform(1.0, 2.0))
+    return results
+
+async def parse_c108(code: str, page: Page) -> list[C108]:
     def extract_bullets(text: str | None) -> list[str]:
         if not text:
             return []
@@ -404,7 +514,7 @@ async def parse_c108(code: str, page: Page) -> pd.DataFrame:
     title = await page.title()
     if '접속장애' in title:
         mylogger.warning(f"c108 스크랩 에러 - {title}")
-        return pd.DataFrame(columns=['날짜', '제목', '작성자', '제공처', '투자의견', '목표가', '내용'])
+        return []
 
     table_locator = page.locator("#tableCmpDetail")
     mylogger.debug((await table_locator.inner_html())[:100])
@@ -423,7 +533,7 @@ async def parse_c108(code: str, page: Page) -> pd.DataFrame:
         mylogger.debug(df)
     except Exception as e:
         mylogger.error(f"테이블 파싱 실패: {e}")
-        return pd.DataFrame(columns=['날짜', '제목', '작성자', '제공처', '투자의견', '목표가', '내용'])
+        return []
 
     row_count = df.shape[0]
     mylogger.debug(f"행의 개수: {row_count}")
@@ -450,33 +560,22 @@ async def parse_c108(code: str, page: Page) -> pd.DataFrame:
     df['내용'] = contents
     mylogger.debug(df)
 
-    return df
+    records = df.replace({np.nan: None}).to_dict(orient="records")
+    for record in records:
+        record['코드'] = code
+    return [C108(**item) for item in records]
 
+async def parse_c108_many(codes: list[str], page:Page) -> list[C108 | None]:
+    results: list[C108|None] = []
 
-T = dict | dict[str, pd.DataFrame] | pd.DataFrame | None
-
-
-async def parse_many(codes: list[str], parser: Callable[[str, Page], Awaitable[T]]) -> dict[str, T]:
-    results: dict[str, T] = {}
-    async with async_playwright() as p:
-        browser = await p.chromium.launch(headless=True)
-        context = await browser.new_context(user_agent=helper.get_random_user_agent(),
-                                            locale="ko-KR")
-        for code in codes:
-
-            print(f"Parsing.. {code} / {parser.__name__.split('_')[1]}")
-
-            page = await context.new_page()
-            try:
-                result = await parser(code, page)
-                results[code] = result
-            except Exception as e:
-                mylogger.error(f"Error while parsing {code}: {e}")
-                results[code] = None
-            finally:
-                await page.close()
-                await asyncio.sleep(random.uniform(3.0, 5.0))
-
-        await context.close()
-        await browser.close()
+    for code in codes:
+        print(f"Parsing.. {code} / c108")
+        try:
+            result = await parse_c108(code, page)
+            results.extend(result)
+        except Exception as e:
+            mylogger.error(f"Error while parsing {code}: {e}")
+            results.append(None)
+        finally:
+            await asyncio.sleep(random.uniform(1.0, 2.0))
     return results
